import pyglet
import graph
import random
import math

# COLORS AVAILABLE
#UNKNOWN
#RED
#BLUE
#GREEN
#YELLOW
#L_BLUE
#D_GREEN
#BROWN
#PINK

class VMBoard(object):

	module_padding= 5
	placement_area= 10

	module_block_size= 3
	pixelsInMemoryBlock= 10	
	

	# INIT
	#
	def __init__(self, module_graph, debug=False):
		self.debug=debug
		
		self.labels_showing= True
		
		self.dim= 0
		self.camera= (0,0)
		self.center_module= None
		
		self.mem_width= 60
		self.mem_height= 60
		self.module_graph= module_graph
		self.module_meta_data= {}		# Module: { Location, [ Pin, ... ], { Pin = [ Pin, ...] }}
		self.module_draw_data= {}		# Module: { (ScreenPosition, Current_Color, Sprite, LabelSprite) }
		self.wirepath_vertices= []		# the vertices for eacch wire
		self.wire_batches= []			# the batches for drawing wires

		self.piece_draw_data= []		# [ (Name, Type, Module, Sprite), ... ]
		self.ogp_modules = pyglet.graphics.OrderedGroup(0)
		self.ogp_labels = pyglet.graphics.OrderedGroup(1)
		self.ogp_pieces = pyglet.graphics.OrderedGroup(2)
		

		self.meta_init(module_graph)
		
		for module in module_graph.nodes():
			self.module_draw_data[module]= (None, "", None, None, None)
		
		self.module_imgs = {}
		self.module_imgs["UNKNOWN"]= pyglet.image.load('images/unknown.bmp')
		self.module_imgs["RED"]= pyglet.image.load('images/red.bmp')
		self.module_imgs["BLUE"]= pyglet.image.load('images/blue.bmp')
		self.module_imgs["GREEN"]= pyglet.image.load('images/green.bmp')
		self.module_imgs["YELLOW"]= pyglet.image.load('images/yellow.bmp')
		self.module_imgs["L_BLUE"]= pyglet.image.load('images/light_blue.bmp')
		self.module_imgs["D_GREEN"]= pyglet.image.load('images/dark_green.bmp')
		self.module_imgs["BROWN"]= pyglet.image.load('images/brown.bmp')
		self.module_imgs["PINK"]= pyglet.image.load('images/pink.bmp')
		
		self.module_imgs["LABEL"]= pyglet.image.load('images/label.bmp')
		
		self.piece_imgs= {}
		self.piece_imgs["PLAYER1"]= pyglet.image.load('images/player1.bmp')

	#enddef
	

	# HIT TEST
	# tests if a click hits on of the nodes
	# return the node clicked
	def hit_test(self, x, y):
		module_size= (VMBoard.module_block_size * VMBoard.pixelsInMemoryBlock)
		for module in self.module_meta_data:			
			if (module.display == 0):
				continue

			if (self.module_draw_data[module][0][0] < x < self.module_draw_data[module][0][0] + module_size and
				self.module_draw_data[module][0][1] < y < self.module_draw_data[module][0][1] + module_size):
				return module
			#endif
		#endfor
		return None
	#enddef


	
	####################################################
	# DRAWING LOGIC
	#
	#
	
	
	# RENDER
	# Create the sprites and draw everything
	def render(self, x, y, width, height, batch, border=2, space=2):
	
		self.batch= batch
		
		self.dim= (x,y,width,height)
		self.border= border
		self.space= space

		self.rerender()
	#enddef


	# RERENDER
	#
	def rerender(self, moving=False):
		if (self.dim == 0):
			return

		if (self.center_module != None):
			self.camera= (
				int(self.dim[0] + (self.dim[2] / 2)) - ((self.module_meta_data[self.center_module][0][1]) * VMBoard.pixelsInMemoryBlock) - int(VMBoard.pixelsInMemoryBlock/2),
				int(self.dim[1] + (self.dim[3] / 2)) - ((self.mem_height - self.module_meta_data[self.center_module][0][0]) * VMBoard.pixelsInMemoryBlock) - int(VMBoard.pixelsInMemoryBlock/2)
				#int(self.dim[0] - (self.module_meta_data[self.center_module][0][1]) * VMBoard.pixelsInMemoryBlock),
				#int(self.dim[1] - (self.mem_height - self.module_meta_data[self.center_module][0][0]) * VMBoard.pixelsInMemoryBlock)
			)
		#endif

		# draw computers
		self._render_modules(update_modules=(~moving))
		
		# draw wires
		if (moving == False):
			self._render_wires()
		else:
			self.wire_batches= []
			self.wires= []
		#endif
		
		# render pieces
		for piece in self.piece_draw_data:  # [ (Name, Type, Module, Sprite), ... ]
			new_position= (
				self.camera[0] + (self.module_meta_data[piece[2]][0][1]) * VMBoard.pixelsInMemoryBlock,
				self.camera[1] + (self.mem_height - self.module_meta_data[piece[2]][0][0]) * VMBoard.pixelsInMemoryBlock
			)
			piece[3].position= new_position
		#endfor
	#enddef


	# SHOW LABELS
	def show_labels(self, show):
		self.labels_showing= show
		self.rerender()
	#enddef
	

	# RENDER MODULES
	# Add each module to the batch
	def _render_modules(self, update_modules=True):
		for module in self.module_meta_data:
			# self.module_draw_data : { ( ScreenPosition, Current_Sprite_Color, Sprite, Label, LabelSprite) }			
			new_position= (
				self.camera[0] + (self.module_meta_data[module][0][1]) * VMBoard.pixelsInMemoryBlock,
				self.camera[1] + (self.mem_height - self.module_meta_data[module][0][0]) * VMBoard.pixelsInMemoryBlock
			)
			
			if (module.display == 0):
				self.module_draw_data[module] = (new_position, "", None, None, None)
				continue
			#endif
			
			if (update_modules):
				#if (module.name == "Node0"): print(module.name,module.color,module.display,self.module_draw_data[module][1])
				if (module.color in self.module_imgs and module.display == 2 and self.module_draw_data[module][1] != module.color):
					newlabel= self.module_draw_data[module][3]
					if (newlabel == None):
						newlabel= pyglet.text.Label(module.name, width=self.module_imgs["LABEL"].width, height=self.module_imgs["LABEL"].height, color=(0, 255, 0, 255), batch=self.batch, group=self.ogp_labels)
					#endif
					
					if (self.module_draw_data[module][2] != None):
						self.module_draw_data[module][2].delete()
					#endif
					if (self.module_draw_data[module][4] != None):
						self.module_draw_data[module][4].delete()
					#endif
					self.module_draw_data[module] = (
						new_position,
						module.color,
						pyglet.sprite.Sprite(self.module_imgs[module.color], batch=self.batch, group=self.ogp_modules),
						newlabel,
						pyglet.sprite.Sprite(self.module_imgs["LABEL"], batch=self.batch, group=self.ogp_modules)
					)
				elif (module.display == 1 and self.module_draw_data[module][1] != "UNKNOWN"):
					newlabel= self.module_draw_data[module][3]
					if (newlabel == None):
						newlabel= pyglet.text.Label(module.name, width=self.module_imgs["LABEL"].width, height=self.module_imgs["LABEL"].height, color=(0, 255, 0, 255), batch=self.batch, group=self.ogp_labels)
					#endif

					if (self.module_draw_data[module][2] != None):
						self.module_draw_data[module][2].delete()
					#endif
					if (self.module_draw_data[module][4] != None):
						self.module_draw_data[module][4].delete()
					#endif
					
					self.module_draw_data[module] = (
						new_position,
						"UNKNOWN",
						pyglet.sprite.Sprite(self.module_imgs["UNKNOWN"], batch=self.batch, group=self.ogp_modules),
						newlabel,
						pyglet.sprite.Sprite(self.module_imgs["LABEL"], batch=self.batch, group=self.ogp_modules)
					)
				else:
					#if (module.name == "Node0"): print(module.name," 3")
					self.module_draw_data[module] = (
						new_position,
						self.module_draw_data[module][1],
						self.module_draw_data[module][2],
						self.module_draw_data[module][3],
						self.module_draw_data[module][4]
					)
				#endif
			#endif
			
			self.module_draw_data[module][2].position= new_position
			self.module_draw_data[module][3].x= new_position[0] - 8
			self.module_draw_data[module][3].y= new_position[1] - 16
			self.module_draw_data[module][4].position= (new_position[0] - 8, new_position[1] - 16)
			
			if (self.labels_showing):
				self.module_draw_data[module][3].text= module.name
				self.module_draw_data[module][4].visible= True
			else:
				self.module_draw_data[module][3].text= ""
				self.module_draw_data[module][4].visible= False
			#endif
		#endfor		
	#enddef
	

	# RENDER WIRES
	# Add each module to the batch
	def _render_wires(self):
		
		self.wire_batches= []
		self.wires= []
		
		for wirepath_data in self.wirepath_vertices:
			if (wirepath_data[0][0].display == 0 or wirepath_data[0][1].display == 0):
				continue
			#endif
			wirepath= wirepath_data[1]
		
			if (len(wirepath) < 2):
				continue
			#endif
			
			self.wire_batches.append(pyglet.graphics.Batch())
			
			v2i_list= self._wirepath_to_v2i(wirepath)
			self.wire_batches[-1].add(len(wirepath), pyglet.gl.GL_LINE_STRIP, None,
				('v2i', v2i_list),
				('c4B', [0, 255, 0, 255] * len(wirepath))
			)
		#endfor
	#enddef
		

	# DRAW WIRES
	# draw the batches created for wires
	def draw_wires(self):
		for batch in self.wire_batches:
			batch.draw()
		#endfor
	#enddef
	

	# CENTER ON
	# move all sprites so the center is occupied by passed module
	def center_on(self, module):
		self.center_module= module
		self.rerender()
	#enddef
	
	
	# MOVE CAMERA
	# "U","D","L","R"
	def move_camera(self, dir):
		self.center_module= None
		if (dir == "R"):
			self.camera= (self.camera[0]-VMBoard.pixelsInMemoryBlock, self.camera[1])
		elif (dir == "L"):
			self.camera= (self.camera[0]+VMBoard.pixelsInMemoryBlock, self.camera[1])
		elif (dir == "U"):
			self.camera= (self.camera[0], self.camera[1]-VMBoard.pixelsInMemoryBlock)
		elif (dir == "D"):
			self.camera= (self.camera[0], self.camera[1]+VMBoard.pixelsInMemoryBlock)
		#endif
		self.rerender()
	#enddef


	# MOVE CAMERA FREE
	# 
	def move_camera_free(self, x, y):
		self.center_module= None
		self.camera= (self.camera[0]+x, self.camera[1]+y)
		self.rerender(moving=True)
	#enddef
	

	# WIRE PATH TO V2I
	# Private. convert the wirepaths (lists of tuples locations) into v2i strings
	def _wirepath_to_v2i(self, locs):
		ret= []
		for loc in locs:
		 	ret.append(self.camera[0] + int((loc[1]  * VMBoard.pixelsInMemoryBlock) + (0.5 * VMBoard.pixelsInMemoryBlock)))
		 	ret.append(self.camera[1] + int(((self.mem_height - loc[0]) * VMBoard.pixelsInMemoryBlock) + (2.5 * VMBoard.pixelsInMemoryBlock)))
		#endfor
		return ret


	# PLACE PIECE
	# draw the player
	def place_piece(self, piece_type, piece_name, module):
		print("BOARD : PLACE PIECE")
		# determine if it exists now
		for piece in self.piece_draw_data:  # [ (Name, Type, Module, Sprite), ... ]
			if (piece[0] != piece_name or piece[1] != piece_type):
				continue

			print("MOVE PIECE: ", piece[3].position)
			
			# set sprite
			piece= (piece_name, piece_type, module, piece[3])
			print("MOVED PIECE: ", piece[3].position)
			break
		else: # didn't break, so we must add it
			sprite= pyglet.sprite.Sprite(self.piece_imgs[piece_type], batch=self.batch, group=self.ogp_pieces)
			self.piece_draw_data.append((piece_name, piece_type, module, sprite))
		#endfor

		self.rerender()
	#enddef
	

	####################################################
	# CREATION LOGIC
	#
	# Tuples/Values:
	# Direction		= *_dir		  =	int, 0=N/S, 1=E/W
	# Location		= *_loc		  =	( row, col )
	# Pin			= *_pin		  =	( Location, direction )
	# Module Data	= module_meta_data =	{ Location, [ Pin, ... ], { Pin = [ Pin, ...] }, Sprite }
	#	First set of Pins are used first
	#	Second set of pins are what open up when first pin is used
	
	blankval= ' '
	moduleval= '#'
	pinval= '*'
	wirevalH='-'
	wirevalV='|'
	wirevalJ='+'

	
	# META INIT
	# initialze memory representation of board
	def meta_init(self, module_graph):
		row= [VMBoard.blankval] * self.mem_width
		self.board= []
		for row_indx in range(self.mem_height):
			self.board.append(row[:])  # replicate
		
		module_list= []
		#nodes= .nodes
		#print nodes
		for module in module_graph:
			module_list.append((len(module_graph.neighbors(module)), module))
		#endfor
		
		module_list.sort(self._compare_setr)
		self.center_module= module_list[0][1]		

		success= False
		while (success == False):
			self._clear_board()
			success= True		
			done= []
			self.wirepath_vertices= []
			last_module= None

			for module_item in module_list:
				module= module_item[1]
				
				# Place Module
				if (module not in done):
					if (self.debug): print "Placing: ", module.name
					if (not self._place_module(module, last_module)):
						success= False
						break  # if we failed to place, start over
					#endif
					done.append(module)
				#endif
				last_module= module
				
				
				# Place Neighbor and Wire Up
				for n_module in module_graph.neighbors(module):
					
					# draw the node if haven't
					if (n_module not in done):
						if (self.debug): print "\tNeighbor: ", n_module.name
						if (not self._place_module(n_module, module)):
							success= False
							break  # if we failed to place, start over
						#endif
						done.append(n_module)
					#endif

					if ((module,n_module) not in done):
						if (self.debug): print "\tWiring: ", module.name,"==>", n_module.name
						if (self._place_wire(module, n_module) == False):
							success= False
							break
						#endif
						done.append((module,n_module))
						done.append((n_module,module))
					#endif
				#endfor
				if (success == False):
					break
				#endif
			#endfor
		#endwhile
		self.print_board()
	#enddef
	
	
	# CLEAR BOARD
	# Private. Clear the Board
	def _clear_board(self):
		for board_loc_y in range(len(self.board)):
			for board_loc_x in range(len(self.board[board_loc_y])):
				self.board[board_loc_y][board_loc_x]= VMBoard.blankval
	#enddef	
	
	
	# PLACE MODULE
	# Private. Draw the module (# for module, x for pins); rowcol is location, does NOT include pins
	def _place_module(self, module, last_module):
		place_module= self._find_space(module, last_module)
		if (place_module == (-1,-1)):
			return False

		row= place_module[0]
		col= place_module[1]

		# print module
		self.board[row][col : col + VMBoard.module_block_size]= [VMBoard.moduleval] * VMBoard.module_block_size
		self.board[row+1][col : col + VMBoard.module_block_size]= [VMBoard.moduleval] + [module.name[-1]] + [VMBoard.moduleval] * (VMBoard.module_block_size - 2)
		for row_indx in range(row + 2, row + VMBoard.module_block_size):
			self.board[row_indx][col : col + VMBoard.module_block_size] = [VMBoard.moduleval] * VMBoard.module_block_size

		# record location and avail pins
		avail_pins= []
		extra_pins= []
		
		center= (row + 1, col + 1)
		
		#top side
		avail_pins.append((center[0] - 1, center[1]))
		#extra_pins[(center[0] - 1, center[1])] = [ (center[0] - 1, center[1] - 1), (center[0] - 1, center[1] + 1) ]
		
		#bottom side
		avail_pins.append((center[0] + 1, center[1]))
		#extra_pins[(center[0] + 1, center[1])] = [ (center[0] + 1, center[1] - 1), (center[0] + 1, center[1] + 1) ]
		
		#left side
		avail_pins.append((center[0], center[1] - 1))
		#extra_pins[(center[0], center[1] - 1)] = [ (center[0] - 1, center[1] - 1), (center[0] + 1, center[1] - 1) ]
		
		#right side
		avail_pins.append((center[0], center[1] + 1))
		#extra_pins[(center[0], center[1] + 1)] = [ (center[0] - 1, center[1] + 1), (center[0] + 1, center[1] + 1) ]
		
		avail_pins.extend([
			(center[0] - 1, center[1] - 1),
			(center[0] - 1, center[1] + 1),
			(center[0] + 1, center[1] + 1),
			(center[0] + 1, center[1] + 1)
		])

		#self.print_board()

		self.module_meta_data[module]=  (place_module, avail_pins, extra_pins)    # the location and the avail pin locations
		return True
	#enddef
	
	
	# PLACE WIRE
	# Private. Find the best path from one module to another
	# Draws the wire
	def _place_wire(self, module_start, module_end):

		openset= []		# a set is a tuple (f(), rowcol)
		closedset= []	# openset is possible directions, closed set is a visited node

		module_end_loc= self.module_meta_data[module_end][0]
		module_end_loc= (module_end_loc[0] + ((VMBoard.module_block_size-1) / 2), module_end_loc[1] + ((VMBoard.module_block_size-1) / 2))

		if (self.debug):print("Start=", self.module_meta_data[module_start])
		if (self.debug):print("End=", self.module_meta_data[module_end])

		# create the dictionaries
		g_score= {}
		h_score= {}
		f_score= {}
		came_from= {}

		
		for pin in self.module_meta_data[module_start][1]:
			g_score[pin]= 0
			h_score[pin]= self._find_distance(pin, module_end_loc)
			f_score[pin]= h_score[pin]

			openset.append((f_score[pin], pin))
		#endfor
		useExtra= True
		

		attempts= 0
		while (len(openset) > 0 and attempts < 3000):
			
			if (len(openset) == 1 and useExtra == True):
				for pin in self.module_meta_data[module_start][2]:
					g_score[pin]= 0
					h_score[pin]= self._find_distance(pin, module_end_loc)
					f_score[pin]= h_score[pin]

					openset.append((f_score[pin], pin))
				#endfor
				useExtra= False
			#endif
			attempts+= 1
			openset.sort(self._compare_set)
			#if (self.debug):print("Open Set=",openset)

			curr_loc= openset[0][1]
			del(openset[0])
			if (curr_loc in closedset):
				continue

			closedset.append(curr_loc)

			# check if goal - if so, draw and quit
			if curr_loc in self.module_meta_data[module_end][1]:
				n_wirepath= []
				n_wirepath.append((curr_loc[0], curr_loc[1]))
				
				if (len(came_from) == 0):
					print "ERROR - didnt move"
					return False
				self.board[curr_loc[0]][curr_loc[1]]= VMBoard.pinval
				
				if (curr_loc[0] == came_from[curr_loc][0]):
					lastwire= VMBoard.wirevalH
				elif (curr_loc[1] == came_from[curr_loc][1]):
					lastwire= VMBoard.wirevalV
				curr_loc= came_from[curr_loc]

				while (came_from.has_key(curr_loc)):
					n_wirepath.append((curr_loc[0], curr_loc[1]))
					if (curr_loc[0] == came_from[curr_loc][0] and lastwire == VMBoard.wirevalH):
						self.board[curr_loc[0]][curr_loc[1]]= lastwire= VMBoard.wirevalH
					elif (curr_loc[1] == came_from[curr_loc][1] and lastwire == VMBoard.wirevalV):
						self.board[curr_loc[0]][curr_loc[1]]= lastwire= VMBoard.wirevalV
					else:
						self.board[curr_loc[0]][curr_loc[1]]= VMBoard.wirevalJ
						if (lastwire == VMBoard.wirevalV):
							lastwire= VMBoard.wirevalH
						elif (lastwire == VMBoard.wirevalH):
							lastwire= VMBoard.wirevalV

					curr_loc= came_from[curr_loc]
				#endwhile
				n_wirepath.append((curr_loc[0], curr_loc[1]))
					
				self.wirepath_vertices.append(((module_start, module_end), n_wirepath))
				
				self.board[curr_loc[0]][curr_loc[1]]= VMBoard.pinval
				return True
			#endif

			for neighbor_loc in self._get_mem_neighbors(curr_loc, came_from=came_from.get(curr_loc,()), dest_pins=self.module_meta_data[module_end][1]):
				if (neighbor_loc in closedset):
					# we've already covered this ground
					continue

				update= False
				curr_g_score= g_score[curr_loc] + self._find_distance(curr_loc, module_end_loc)
				if (neighbor_loc not in openset):
					h_score[neighbor_loc]= self._find_distance(neighbor_loc, module_end_loc)
					update= True
				elif curr_g_score < g_score[neighbor_loc]:
					update= True  # its in the openQ, but it is better if we come from this angle

				if (update):
					came_from[neighbor_loc]= curr_loc
					g_score[neighbor_loc]= curr_g_score
					f_score[neighbor_loc]= g_score[neighbor_loc] + h_score[neighbor_loc]
					openset.append((f_score[neighbor_loc], neighbor_loc))
				#endif
			#endfor
		#endwhile
		return False
	#enddef
	
	
	# PRINT BOARD
	#
	def print_board(self, filename=""):
		for x in range(len(self.board)):
			str= "["
			for y in range(len(self.board[x])):
				str = str + ("{0}".format(self.board[x][y]))
			print("{0}]".format(str))

		#print file if filename provided
		if (len(filename) > 0):
			out= open(filename,'a')
			for x in range(len(self.board)):
				out.write("[")
				for y in range(len(self.board[x])):
					out.write("{0}".format(self.board[x][y]))
				out.write("]\n")
			out.close()
		#endif
	#enddef
	
	
	# FIND SPACE
	# Private.  Returns a space for module with enough area around, so no modules are adjacent
	# does not include area around module in return coords
	def _find_space(self, module, last_module):
		if (last_module is None):
			return (int(self.mem_height / 2), int(self.mem_width / 2))
		
		find_space= (VMBoard.module_block_size + (2 * VMBoard.module_padding), VMBoard.module_block_size + (2 * VMBoard.module_padding))  # the space we need to find

		placement_area= VMBoard.placement_area
		for attempt in range (0,5):
		
			randrange_row= range(-placement_area,-VMBoard.module_padding)
			randrange_row.extend(range(VMBoard.module_block_size + VMBoard.module_padding, placement_area))

			randrange_col= range(-placement_area,-VMBoard.module_padding)
			randrange_col.extend(range(VMBoard.module_block_size + VMBoard.module_padding, placement_area))

			random_loc= []

			for row in randrange_row:
				for col in randrange_col:
					try_space= (self.module_meta_data[last_module][0][0] + row , self.module_meta_data[last_module][0][1] + col)
					if (self._test_space(try_space, find_space[0], find_space[1]) == True):
						random_loc.append(try_space)
					#endif
				#endfor
			#endfor
			if (len(random_loc) == 0):
				placement_area += 10
				continue
			#endif
			
			test_loc= random_loc[random.randint(0, len(random_loc)-1)]				
			return (test_loc[0]+VMBoard.module_padding, test_loc[1]+VMBoard.module_padding)
		else:
			ret= (-1,-1)
		return ret;
	#enddef
	
	
	# TEST SPACE
	# Private. Test the block is empty (all blankval)
	def _test_space(self, rowcol, height, width):
		row= rowcol[0]
		col= rowcol[1]
		
		teststr= VMBoard.blankval * width
		
		for row_indx in range(row, row + height):
			if (col < 0 or row_indx < 0 or row_indx >= len(self.board)):
				return False
			samplestr= "".join(self.board[row_indx][col : col + width])	
			if (len(samplestr) != len(teststr) or samplestr != teststr):
				return False
		if (self.debug): print "Testing True: ", rowcol, height, width
		return True
	#enddef


	# GET MEM NEIGHBORS
	# Private. get neighbors of rowcol, if filter=True don't return occupied cells
	def _get_mem_neighbors(self, rowcol, came_from=(), dest_pins=[], filter=True):
		ret= []
		possible_neighbors= [
			(rowcol[0] + 1, rowcol[1]),
			(rowcol[0] - 1, rowcol[1]),
			(rowcol[0]    , rowcol[1] + 1),
			(rowcol[0]    , rowcol[1] - 1),
		]
		
		for neighbor in possible_neighbors:
			r= neighbor[0]
			c= neighbor[1]
			if (r < 0 or r >= self.mem_height):
				continue
			if (c < 0 or c >= self.mem_width):
				continue
			
			if (filter):
				#if (self.debug):print("Neighbor=",neighbor)
				if (neighbor not in dest_pins and self.board[r][c] != VMBoard.blankval):
					continue
			if (came_from == neighbor):
				continue
			ret.append(neighbor)
		#endfor
		
		#if (self.debug):print("Get Neighbors for:",rowcol,"=",ret)
		
		return ret
	#enddef


	# FIND DISTANCE
	# Private. Sqr Distance between two locations
	def _find_distance(self,a,b):
		return math.pow(a[0]-b[0],2) + math.pow(a[1]-b[1],2)
	#enddef
	

	# COMPARE SET
	# Private. Compare two tuples based on first
	def _compare_set(self,a,b):
		return cmp(a[0],b[0])
	#enddef
	def _compare_setr(self,a,b):
		return cmp(b[0],a[0])
	#end
#enddefclass