


# VMSystem
# 
class VMSystem:
	# INIT
	# proc_list is a VMProcList class, used to control what programs are running
	# VMSystem does not do anything regarding drawing
	def __init__(self, proc_list, window, board, starting_module):
		self._proc_list= proc_list		
		self._proc_list.add_process("System",1,pid=1)		

		self.window= window		
		self.board= board
		
		self.board.center_on(starting_module)
		self.board.place_player(starting_module)
		
		self.commands= {}  # Command: { Type, Function, Help } 
		self.commands["anlz"]= ("OneShot", self._cmd_analyze, "[Node] Analyze a neighboring Node")
		self.commands["move"]= ("OneShot", self._cmd_move, "[Node] Move to a neighboring Node")
		self.commands["tags"]= ("OneShot", self._cmd_tags, "[On|Off] removes names of nodes on Display")
		self.commands["help"]= ("OneShot", self._cmd_help, "This command")
	#enddef
	
	
	# TICK
	# 
	def tick(self, dt):
		pass
	#enddef
	
	
	# WELCOME SCREEN
	#
	def welcome_screen(self):
		self.printout("Welcome Virtual Mercenary")
		self.printout("Type 'help' to get a list of availble commands")
		self.printout("Click on a Node to have it entered into the entry prompt automatically")
	#enddef
	
	
	# SET OUTPUT CALLBACK
	# function to call when we have some output
	def set_print_callback(self, funcCB):
		self.printout= funcCB
	#enddef


	# READ INPUT
	# parse input from user
	def read_input(self, instr):
		instr.replace("\r","") 
		instr.replace("\n","")
		
		parms= instr.split()
		if (len(parms) > 0 and parms[0] in self.commands):
			self.commands[parms[0]][1](parms[1:])
		else:
			self.printout("Unknown command or filename")
			return

	#enddef
	
	
	
	################################################
	#   COMMANDS
	#
	
	
	# COMMAND - MOVE
	#
	def _cmd_move(self, commandArgs):
		if (len(commandArgs) <= 0):
			self.printout("move: requires node")
		#endif
		
		move_module= self.board.get_module_by_name(commandArgs[0])
		if (move_module == None):
			self.printout("move: Unrecognized Node: " + commandArgs[0])
			return
		#endif
		
		if (self.board.move_player(move_module)):
			self.printout("Moving To Node: " + move_module.name)
			self.board.center_on(move_module)
		#endif		

	#enddef
	
	
	# COMMAND - ANALYZE
	#
	def _cmd_analyze(self, commandArgs):
		if (len(commandArgs) <= 0):
			self.printout("anlz: requires node")
		#endif
		
		anlz_module= self.board.get_module_by_name(commandArgs[0])
		if (anlz_module == None):
			self.printout("anlz: Unrecognized Node: " + commandArgs[0])
			return
		#endif
		
		self.printout("Analyzing Node: " + anlz_module.name)
		self.board.analyze_module(anlz_module)
	#enddef



	# COMMAND - TAGS
	# print out the help
	def _cmd_tags(self, commandArgs):
		if (len(commandArgs) <= 0):
			self.printout("tags: requires on/off")
			return 
		#endif
		
		commandArgs[0]= commandArgs[0].lower()
		if (commandArgs[0] == "on" or commandArgs[0] == "true"):
			self.board.show_labels(True)
			self.printout("Tags: on")
		elif (commandArgs[0] == "off" or commandArgs[0] == "false"):
			self.board.show_labels(False)
			self.printout("Tags: off")
		else:
			self.printout("tags: requires on/off")
		#endfor
	#enddef
	
	
	
	# COMMAND - HELP
	# print out the help
	def _cmd_help(self, commandArgs):
		self.printout("Available Commands:")
		for command in self.commands:
			if (command == "help"):
				continue
			self.printout("  " + command + " - " + self.commands[command][2])
		#endfor
		self.printout("  help - " + self.commands["help"][2])
	#enddef
#endclass