import pyglet
import random
import vm_components

from pyglet.window.key import MOTION_UP, MOTION_RIGHT, MOTION_DOWN, MOTION_LEFT


# RECTANGLE
# Class.
class Rectangle:
	'''Draws a rectangle into a Globs.batch.'''
	def __init__(self, x1, y1, x2, y2, batch, color=[200, 200, 220, 255]):
		if (batch is None):
			batch= VMWindow.batch
		x2= x1 + x2
		y2= y1 + y2
		self.vertex_list = batch.add(4, pyglet.gl.GL_QUADS, None,
			('v2i', [x1, y1, x2, y1, x2, y2, x1, y2]),
			('c4B', color * 4)
		)
	#enddef
#endclass


#######################################
# VM WINDOW
#
class VMWindow(pyglet.window.Window):
	batch= pyglet.graphics.Batch()
	board_batch= pyglet.graphics.Batch()
	grid_batch= pyglet.graphics.Batch()
	
	def __init__(self, vmprompt, vmconsole, vmproclist, vmboard, *args, **kwargs):
		super(VMWindow, self).__init__(600, 500, caption='virt_merc')
		self.drawn= True
		
		# Background
		self._render_background()
		
		# Prompt
		self.prompt= vmprompt
		self.prompt.render(0, 0, 450, VMWindow.batch)
		
		# Console
		self.console= vmconsole
		self.console.render(0, self.prompt.dim[3], self.prompt.dim[2], 200, VMWindow.batch)
		
		# Proc List
		self.proc_list= vmproclist
		self.proc_list.render(
			self.prompt.dim[2],				# right of console
			0,								# against bottom
			self.width - self.prompt.dim[2],							# fill to the rest of the screen
			250,							# 250 height
			VMWindow.batch
		)
		
		# Board
		self.board= vmboard
		self.board.render(
			0,								# right of the console
			self.proc_list.dim[1] + self.proc_list.dim[3],		# above the proclist
			self.width,					# fill to the rest of the screen
			self.height - self.proc_list.dim[3],
			VMWindow.board_batch
		)
		
		self.focus = None
		self.set_focus(self.prompt)
		
		self.last_over= ""
	#enddef
	
	
	# SET FOCUS
	#
	def set_focus(self, focus):
		if self.focus:
			self.focus.caret.visible = False
			self.focus.caret.mark = self.focus.caret.position = 0

		self.focus = focus
		if self.focus:
			self.focus.caret.visible = True
			self.focus.caret.mark = 0
		self.focus.caret.position = len(self.focus.document.text)
	#enddef
	
	
	# SET INPUT CALLBACK
	# set the function we call when we get an input string
	def set_input_callback(self, funcCB):
		self.input_call= funcCB
	#enddef
	
	
	### EVENTS ###
	
	# ON DRAW
	# 
	def on_draw(self):			
		pyglet.gl.glClearColor(0, 0, 0, 1)
		self.clear()
		
		VMWindow.grid_batch.draw()
		self.board.draw_wires()
		VMWindow.board_batch.draw()		
		VMWindow.batch.draw()
		
	#enddef
	
	# ON RESIZE
	#
	def on_resize(self, width, height):
		super(VMWindow, self).on_resize(width, height)
		return

	# ON MOUSE MOTION
	#
	def on_mouse_motion(self, x, y, dx, dy):
		
		return

	# ON MOUSE SCROLL
	#
	def on_mouse_scroll(self, x, y, dx, dy):
		if (self.console.hit_test(x,y)):
			self.console.scroll(dy)
		return

	# ON MOUSE RELEASE
	#
	def on_mouse_release(self, x, y, button, modifiers):	
		self.board.rerender()
		return

	# ON MOUSE PRESS
	#
	def on_mouse_press(self, x, y, button, modifiers):
		hit_module= self.board.hit_test(x,y)
		if (hit_module != None):
			if (self.last_over != hit_module.name):
				self.focus.caret.on_text(" " + hit_module.name)
		return
	
	# ON MOUSE DRAG
	#
	def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
		self.drawn= True
		self.board.move_camera_free(dx, dy)
		self.on_draw()
		return
	
	# ON TEXT
	#
	def on_text(self, text):
		if (text == "\r" or text == "\n"):
			self.input_call(self.prompt.get_text())
		elif self.focus:
			self.focus.caret.on_text(text)
	#enddef
	
	
	# ON TEXT MOTION
	#
	def on_text_motion(self, motion):
		if (motion == MOTION_UP):
			self.board.move_camera("U")
		elif (motion == MOTION_DOWN):
			self.board.move_camera("D")
		elif (motion == MOTION_RIGHT):
			self.board.move_camera("R")
		elif (motion == MOTION_LEFT):
			self.board.move_camera("L")
		elif self.focus:
			self.focus.caret.on_text_motion(motion)
	#enddef


	# ON TEXT MOTION
	#		
	def on_text_motion_select(self, motion):
		return
		
	# ON KEY PRESS
	#
	def on_key_press(self, symbol, modifiers):
		if (symbol == pyglet.window.key.ESCAPE):
			pyglet.app.exit()
#		elif (symbol == pyglet.window.key.ENTER):
			#self.console.send_text(self.focus.get_text())
	#enddef


	# RENDER BACKGROUND
	#
	def _render_background(self):
		# draw grid
		grid= True
		grid_size= 20
		
		if (grid):
			distance= -5
			while (distance < self.width):
				
				# up
				VMWindow.grid_batch.add(2, pyglet.gl.GL_LINE_STRIP, None,
					('v2i', [distance, -5, distance, self.height + 5]),
					('c4B', [255, 0, 0, 255] * 2)
				)
				
				distance+= grid_size
				
				#down
				VMWindow.grid_batch.add(2, pyglet.gl.GL_LINE_STRIP, None,
					('v2i', [distance, self.height + 5, distance, -5]),
					('c4B', [255, 0, 0, 255] * 2)
				)
				
				distance+= grid_size
			
			#endwhile
			
			distance= -5
			while (distance < self.height):
				
				# right
				VMWindow.grid_batch.add(2, pyglet.gl.GL_LINE_STRIP, None,
					('v2i', [-5, distance, self.width + 5, distance]),
					('c4B', [255, 0, 0, 255] * 2)
				)
				
				distance+= grid_size
				
				# left
				VMWindow.grid_batch.add(2, pyglet.gl.GL_LINE_STRIP, None,
					('v2i', [self.width + 5, distance, -5, distance]),
					('c4B', [255, 0, 0, 255] * 2)
				)
				
				distance+= grid_size
			
			#endwhile
		#endif
	#enddef
#endclass